package cz.kebrt.unitcheck.console;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;

import cz.kebrt.unitcheck.api.UnitChecker;
import cz.kebrt.unitcheck.config.Config;
import cz.kebrt.unitcheck.config.ConfigException;
import cz.kebrt.unitcheck.config.PathPrinterConfig;
import cz.kebrt.unitcheck.config.TestConfig;
import cz.kebrt.unitcheck.util.ContextClassPathFactory;

/**
 * Command-line UnitCheck tool. Options are represented by the {@link Options} class.
 *
 * @author michal
 */
public final class ConsoleUnitCheckRunner {

    private static final String LIB_DIR = "lib";

    @SuppressWarnings("unused")
    private static final Logger log = Logger.getLogger(UnitChecker.class);

    private ConsoleUnitCheckRunner() { }

    /**
     * Runs UnitCheck.
     *
     * @param args command-line options (will be parsed by args4j)
     */
    public static void main(String[] args) {
        Options options = new Options();
        CmdLineParser parser = new CmdLineParser(options);

        try {
            parser.parseArgument(args);
        } catch (CmdLineException e) {
            System.err.println(e.getMessage());
            parser.printUsage(System.err);
            return;
        }

        try {
            Config config = createConfig(options);
            PathPrinterConfig pathConfig = new PathPrinterConfig(options.isTraceJRE(),
                    options.isPrintBytecode());
            UnitChecker uc = new UnitChecker(config);
            uc.addListener(new ConsoleUnitCheckListener(System.out, pathConfig,
                    !options.isNoExecutionHistory()));
            uc.run();
        } catch (ConfigException e) {
            System.err.println(e.getMessage());
        }
    }

    /**
     * Create a configuration from the command line options.
     */
    private static Config createConfig(Options options) throws ConfigException {
        Config config = new Config();

        if (options.getVmClassPath() != null) {
            List<String> cp = Arrays.asList(options.getVmClassPath().split(":"));
            config.setVMClasspath(cp);
            ClassLoader cl = ContextClassPathFactory.create(cp);
            Thread.currentThread().setContextClassLoader(cl);
        }

        TestConfig tc = new TestConfig(options.getTestClass(), options.getTestMethod());
        config.setOneTest(tc);
        config.setLibDir(String.format("%s/%s", options.getBaseDir(), LIB_DIR));
        config.setFilterJUnitTrace(!options.isNoFilterJUnitTrace());
        config.setHaltOnError(!options.isNoHaltOnError());
        config.setJPFConfigFile(options.getJPFConfigFile());

        List<String> sourcePath = null;
        if (options.getVmSourcePath() != null) {
            sourcePath = Arrays.asList(options.getVmSourcePath().split(":"));
        } else {
            sourcePath = new ArrayList<String>();
        }
        config.setVMSourcepath(sourcePath);

        return config;
    }

}
